# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - MoinShare fragment parser

    @copyright: 2011, 2012 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin import wikiutil

try:
    from hashlib import sha1
except ImportError:
    from sha import new as sha1

Dependencies = ["pages"]

# Parser support.

class Parser:

    """
    Format shared content fragments of the form...

    {{{#!MoinShare ref=fragment format=content-type

    Content...
    }}}
    """

    def __init__(self, raw, request, **kw):

        """
        Initialise the parser with the given 'raw' data, 'request' and any
        keyword arguments that may have been supplied.
        """

        self.raw = raw
        self.request = request

        attrs, msg = wikiutil.parseAttributes(request, kw.get("format_args", ""))

        # The attributes returned from the formatting arguments are encoded like
        # strings.

        quotes = '"' + "'"

        # Get any reference to another fragment.

        self.ref = (attrs.get("ref") or '').strip(quotes)
        self.format_type = (attrs.get("format") or '').strip(quotes)

    def getFragmentIdentifier(self):

        "Make a unique identifier for the fragment."

        return sha1(self.raw).hexdigest()

    def format(self, fmt):

        "Format a fragment using the given formatter 'fmt'."

        request = self.request
        page = request.page
        _ = request.getText

        request.write(fmt.anchordef(self.getFragmentIdentifier()))

        request.write(fmt.div(1, css_class="moinsharefragment"))
        request.write(self.formatText(self.raw, fmt))
        request.write(fmt.div(0))

    def getFormat(self):

        "Get the format used on this page."

        return self.format_type

    def getParserClass(self, format):

        """
        Return a parser class for the given 'format', returning a plain text
        parser if no parser can be found for the specified 'format'.
        """

        cfg = self.request.cfg

        try:
            return wikiutil.searchAndImportPlugin(cfg, "parser", format or "wiki")
        except wikiutil.PluginMissingError:
            return wikiutil.searchAndImportPlugin(cfg, "parser", "plain")

    def formatText(self, text, fmt):

        """
        Format the given 'text' using the specified formatter 'fmt'.
        """

        request = self.request
        page = fmt.page = request.page
        format = self.getFormat()

        # Suppress line anchors.

        parser_cls = self.getParserClass(format)
        parser = parser_cls(text, request, line_anchors=False)

        # Fix lists by indicating that a paragraph is already started.

        kw = {}
        if not format or format == "wiki":
            kw["inhibit_p"] = True

        return request.redirectedOutput(parser.format, fmt, **kw)

# vim: tabstop=4 expandtab shiftwidth=4
